; Temperature Measurement System  ver. 2.15 / 4. 4. 2000
; Copyright  (c) 1999, 2000, Pavel Kaspar
;
; Komercni pouziti pouze s vyslovnym souhlasem autora

                NAME	TMS
MainVer         EQU     '2'          ; Cislo verze programu
SubVer          EQU     '15'         ; Podverze programu

DATEsep         EQU     '.'          ; Oddelovac v Datumu
TIMEsep         EQU     ':'          ; Oddelovac v Case
DECpoint        EQU     '.'          ; Desetinna carka
CMDprefix       EQU     '*'          ; Znak bezprostredne predchazejici prikaz

PCON		EQU	087h	     ; Tato verze ASM51 nezna PCON

BR1200          EQU     0E8h         ; Casovaci konstanty pro Baud Rate
BR2400          EQU     0F4h
BR9600          EQU     0FDh
TH0_Val         EQU     04Ch         ; Konstanty pro naplneni Casovace 0
TL0_Val         EQU     000h         ; 50 ms; 0B8h pro 20 ms

UpCase          EQU     0DFh         ; Konstanta pro prevod mala-velka pismena
RS_Flag         EQU     0D1h         ; Bit, ktery se pouziva pro rozliseni
                                     ; vystupu na LCD (0)/RS (1) = Bit1 z PSW
InvalTMP        EQU     03Fh         ; Konstanta pro neplatnou teplotu
PocetOkruhu     EQU     008h         ; Pocet okruhu - pro cykly
MaxLogH         EQU     16*1024/256  ; Maximalni + 1 (tedy 1. neplatna) adresa
                                     ; v Logu Udalosti
MaxLogL         EQU     000h         ; Musi byt vzdy 0
MinLogH         EQU     000h         ; Min. adresa = konec rezervovane oblasti
MinLogL         EQU     000h
PageSize        EQU     064          ; Velikost stranky pro Page Write

; ************************************************************ Mapa pameti RTC
; 00h - 0Fh       RTC
; 10h             Rok
; 11h             Pocet zapnuti
; 12h - 17h       Sec, Min, Hour, Day, Month, Year - Posledni zapnuti
; 18h - 19h       Adresa prvniho volneho zaznamu v Logu Udalosti (L, H)
; 1Ah - 1Fh       Free
; 20h - ..        Senzory
;                 MinTempH, MinTempL, Min, Hour, Day, Month,
;                 MaxTempH, MaxTempL, Min, Hour, Day, Month
; ******************************************************* Zaznam Logu Udalosti
; 00h             YYMM MMDD
; 01h             DDDH HHHH
; 02h - 0Fh       7 dvojic Bytu ve formatu 88nn nnnn nnnn nnnn kde
;                 n je reprezentace teploty n-teho okruhu (n=0..6) a
;                 8 jsou rozprostrene bity teploty 8. okruhu
;                 (MSB je B7 na adrese 02h, LSB B6 na adrese 0Fh)
; Vzhledem k tomu, ze Datum v Byte 0 nemuze nabyvat vsech 256 hodnot, lze Log
; vyuzit i pro ukladani jinych udalosti s nasledujicimi vyhrazenymi hodnotami:
; FFh = 1111 1111 - Prazdny zaznam
; 00h = 0000 0000 - Pokracovani predchoziho zaznamu
;       xx11 01xx - Rezervovano
;       xx11 10xx - Rezervovano
;       xx11 11xx - Jina udalost
; ****************************************************** Mapa pameti Procesoru

                DSEG

                ORG     0010h       ; RB0 - pracovni, RB1 - rezerva pro INT
Sec:            DS      1
Min:            DS      1
Hour:           DS      1
Day:            DS      1           ; YYTT DDDD
Month:          DS      1           ; WWWT MMMM
Year:           DS      1           ; Posledni prestupny rok v BCD
Counter:        DS      1           ; Citac pro Casovac 0 (pro Konverzi)
Okruh:          DS      1           ; Cislo mereneho okruhu
OkruhCnv:       DS      1           ; Konvertovane cislo -> Bit Portu
LastLogHour:    DS      1           ; Posledni hodina zaznamenana do Logu
LogAddrL:       DS      1           ; Adresa Prvniho volneho zaznamu v Logu
LogAddrH:       DS      1
Stat174:        DS      1           ; Pro ulozeni posledniho vystupu na 174
KBD:            DS      1           ; Stav Klavesnice
KBDPom:         DS      1           ; Pomocny predchozi stav
CMDChar:        DS      1           ; Posledni Znak prikazu obdrzeny po RS
Logged:         DS      1           ; Stav prihlaseni, <> 0 Prihlasen
Node:           DS      1           ; Aktualni Uroven pro SearchROM
OldNode:        DS      1           ; Stare Rozcesti z predchoziho SearchROM
NewNode:        DS      1           ; Nove Rozcesti pro nasledujici SearchROM
ROMBuff:        DS      9           ; Buffer pro ulozeni LaseredROM

                ORG     0030h
SenArray:                           ; Pole namerenych hodnot vsech senzoru
TempH:          DS      1           ; Teplota v setinach stupnu v HEX
TempL:          DS      1           ; s posunutou nulou na -50 stupnu
MinTempH:       DS      1           ; Je mozno ulozit teploty -50 .. +111
MinTempL:       DS      1
MaxTempH:       DS      1
MaxTempL:       DS      1
                DS      42          ; 7 * 6 pro senzory 1 - 7


                ORG     0060h       ; Pocatek zasobniku
Stack:          DS      0020h       ; Max. hloubka vnoreni

                CSEG

                ORG     0000h
PowerOn:        CLR     A
                SJMP    On_Reset

                ORG     0003h
ExtInt0:	RETI

                ORG     000Bh
Timer0Int:      MOV     TH0, #TH0_Val    ; Naplneni Casovace 0
                MOV     TL0, #TL0_Val
                INC     Counter
                RETI

                ORG     0013h
ExtInt1:        RETI

                ORG     001Bh
Timer1Int:      RETI

                ORG     0023h
SerialInt:      RETI

                ORG     002Bh
Timer2Int:      RETI


                ORG     0030h

On_Reset:       MOV	SP, #(Stack-1)	; Inicializace zasobniku
                MOV     PSW, A          ; Banka 0, v A je 0
                MOV     Stat174, A      ; 0 = Vse vypnuto
                CALL    Out174          ; Inicializace 174
                MOV     CMDchar, A      ; Vynuluj
                MOV     SCON, #01010011b; Mod 1; 1 start,8 datovych,1 stop bit
		MOV	TMOD, #00100001b; C/T0 - mod 1, casovac 16 bitu
					; C/T1 - mod 2, casovac pro Serial
                MOV     TH1, #BR9600    ; Prenosova rychlost
                MOV     TL1, #BR9600
                MOV     PCON, #00000000b; Nasobitel SMOD = 0
		MOV	TCON, #01010101b; C/T0,1 povolen; EI0,1 reag. na hranu
                SETB    TI              ; Pro vyslani 1. znaku, jinak DeadLock
                CLR     RI              ;
		MOV	IE, #00000010b	; Disable, pripraveno pro T0
		MOV	IP, #00000001b  ; EI0 ma prioritu 1 - RTC

                MOV     A, #04h         ; Nastaveni PCF8583, Alarm enable
                MOV     R2, #00h        ; Ridici registr (Control/Status)
                CALL    WriteRTC        ; Pri Alarm disable pousti 1 Hz na INT
                MOV     A, #00h         ; Nastaveni PCF8583, No INT
                MOV     R2, #08h        ; Alarm registr (Alarm Control)
                CALL    WriteRTC
                CALL    RTC             ; Zjisti aktualni cas

                MOV     R2, #11h        ; 11 - Pocet Zapnuti
                CALL    ReadRTC         ; uloz do A
                INC     A               ; Zvys o 1
                CALL    WriteRTC        ; uloz do RTC

                DEC     R2              ; 10 - Aktualni Rok
                CALL    ReadRTC         ; uloz do A
                MOV     R0, A           ; a R0
                ACALL   Prestupny       ; Vypocti Posledni prestupny rok
                MOV     Year, A         ; a uloz jej do Year

                MOV     A, R0           ; Zmenil se rok ?
                CALL    BCD_HEX         ; Porovnej Aktualni rok z RAM a z RTC
                ANL     A, #03h         ; Jen posledni 2 bity
                MOV     R0, A           ; do R0 predpokladany rok podle RAM
                MOV     A, Day
                ANL     A, #0C0h        ; YYTT DDDD
                RL      A
                RL      A               ; v A rok podle RTC
                CLR     C
                SUBB    A, R0
                JNC     UlozRok         ; Nebyl prenos => A >= R0
                MOV     A, Year         ; Byl prenos => doslo k protoceni
                ADD     A, #04          ; Zvys posledni prestupny rok o 4
                DA      A               ; Dekadicka korekce
                MOV     Year, A
UlozRok:        ACALL   SaveYear        ; Ulozi Aktualni rok do RAM v RTC

UlozPosZap:                             ; Ulozi Cas posledniho zapnuti do RTC
                MOV     R0, #Sec        ; Adresa Sec v RAM
                MOV     R2, #12h        ; Adresa Sec v RTC
                MOV     B, #5           ; Sec, Min, Hour, Day, Month
UPosZap0:
                MOV     A, @R0          ; Precti z pameti do A
                CALL    WriteRTC        ; Uloz z A do RTC
                INC     R0
                INC     R2
                DJNZ    B, UPosZap0

                MOV     R2, #10h        ; A nyni Rok
                CALL    ReadRTC         ; Precti z RTC do A
                MOV     R2, #17h        ; Adresa Roku
                CALL    WriteRTC        ; Uloz z A do RTC


                CALL    InitLCD
                CLR     RS_Flag
                ACALL   ZobrCopyright   ; Zobraz Copyright na LCD 

                CALL    Start_Conv      ; Prvni Konverze, Nastavi Timer0
                CALL    RTC             ; Nacti cas prvni Konverze

                ACALL   InitLog         ; Inicializace Logu Udalosti
                ACALL   InitModem       ; Init modemu, vcetne Logged := 0

                MOV     B, #8*6         ; Smaze 8*6=48 bunek pameti
                MOV     R0, #SenArray   ; tj. pamet teplot
SmazMem:        MOV     @R0, #InvalTMP
                INC     R0
                DJNZ    B, SmazMem

PomWait:                                ; Cekej na dokonceni Konverze
                CLR     A
                CALL    DelayA          ; Pomocne cekani - i pro Copyright
                CALL    DelayA
                CALL    ClrLCD          ; Smaz Display

; -------------------------------------------------------------- Hlavni smycka
Main:
                CALL    CursorOff

                MOV     Okruh, #00
                MOV     B, #PocetOkruhu
MainPrecti:
                MOV     R1, #00h        ; Zajima mne TempH = nulovy posuv
                CALL    OffsetSA        ; Adresa TempH z Okruhu
                MOV     @R0, #InvalTMP  ; Pro pripad nepritomnosti cidla
                CALL    ReadTMP         ; Nacte teplotu do @R0
                CALL    CompTMP         ; Provna teploty Min a Max
                INC     Okruh
                DJNZ    B, MainPrecti

                CALL    Start_Conv      ; Nova konverze, povoli preruseni
                CALL    RTC             ; Aktualni cas

MainLogTst:
                MOV     A, LastLogHour
                CJNE    A, Hour, MainLog
                SJMP    MainZobr
MainLog:        MOV     LastLogHour, Hour
                ACALL   UpDateLog
                ACALL   InitModem       ; Inicializace Modemu, kazdou hodinu
                                        ; vcetne Logged := 0
MainZobr:
                MOV     A, #0E0h
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
                ACALL   ZobrTIME
                MOV     A, #0D4h
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
                ACALL   ZobrDATE

                CALL    TestKBD
                MOV     A, KBD
                ANL     A, #03h         ; Nejnizsi 2 bity
                CJNE    A, #0, MainMin
                ACALL   ZobrCurT        ; RS_Flag stale shozen
                SJMP    MainCekej
MainMin:
                CJNE    A, #1, MainMax
                ACALL   ZobrMinT        ; RS_Flag stale shozen
                SJMP    MainCekej
MainMax:
                CJNE    A, #2, MainCopyr
                ACALL   ZobrMaxT        ; RS_Flag stale shozen
                SJMP    MainCekej
MainCopyr:                              ; Jsou-li stisknuta obe -> Copyright
                ACALL   ZobrCopyright   ; RS_Flag stale shozen
                SJMP    PomWait         ; Potom pokracuje do Main

MainCekej:                              ; Cekej na dokonceni Konverze
                CLR     C
                MOV     A, Counter
                SUBB    A, #10          ; 10 * 50 ms = 500 ms
                JNC     Main            ; Nebyl C => Cekej >= 10

                JNB     RI, MainCekej   ; Zadny znak -> Cekej na Konverzi
                CLR     RI              ; jinak jej zpracuj
                MOV     A, SBUF         ; Aktualni uloz do A
                XCH     A, CMDchar      ; Predchozi znak do A, aktualni do CMD
                CJNE    A, #CMDprefix, MainCekej ; Cekej na CMD prefix

                SETB    RS_Flag
                MOV     A, Logged       ; Test uspesneho loginu
                JNZ     IsLogged
NotLogged:                              ; Nikdo neni zalogovan, otestuj LogCMD
                MOV     A, CMDchar
                CJNE    A, #'@', MainCekej ; Neni prikaz pro Login => skoc
                INC     Logged          ; Nastav priznak uspesneho Loginu
                CALL    ZobrCopyright
                SJMP    MainCRLF

IsLogged:
                ACALL   ExecRSCmd       ; Je zalogovan, proved prikaz
MainCRLF:
                CALL    ZobrCRLF
                SJMP    MainCekej

; ----------------------------------------------------------- Prikaz po RS-232

ExecRSCmd:
                MOV     A, CMDchar
                CJNE    A, #'c', MaxT_RS
                AJMP    ZobrCurT        ; Aktualni teplota
MaxT_RS:
                CJNE    A, #'M', MinT_RS
                AJMP    ZobrMaxT        ; Max teplota
MinT_RS:
                CJNE    A, #'m', Detail
                AJMP    ZobrMinT        ; Min teplota
Detail:
                CJNE    A, #'d', LaseredROM
                AJMP    ZobrRSDetail    ; Detailni vypis teplot
LaseredROM:
                CJNE    A, #'n', EETmpLog
                AJMP    ZobrRSROM       ; Vypis LaseredROM senzoru
EETmpLog:
                CJNE    A, #'l', EEListing
                AJMP    ZobrLog         ; Vypis Log Udalosti na RS
EEListing:
                CJNE    A, #'L', Copyr
                AJMP    ZobrEEListing   ; Vypis Log Udalosti na RS
Copyr:
                CJNE    A, #'?', OptoStat
                CALL    ZobrCopyright   ; Zobraz Copyright
                MOV     DPTR, #Help     ; Zobraz Help, vcetne CRLF
		JMP     WrStr0

OptoStat:
                CJNE    A, #'s', OptoSt10x ; Zobraz stav vstupu OPTO1 a 2
OptoSta0:       MOV     DPTR, #Opto1Str  
		CALL    WrStr0
                MOV     A, #'0'
                JNB     P3.4, OptoSta1
                INC     A               ; Bit Nastaven -> 0 zmen na 1
OptoSta1:       ACALL   WriteChar

                MOV     DPTR, #Opto2Str  
		CALL    WrStr0
                MOV     A, #'0'
                JNB     P3.3, OptoSta2
                INC     A               ; Bit Nastaven -> 0 zmen na 1
OptoSta2:       AJMP    WriteChar

OptoSt10x:
                CJNE    A, #'S', CmdEcho; Zobraz 10x stav vstupu OPTO1 a 2
                MOV     B, #10
OptoSt11:       ACALL   OptoSta0
                CALL    ZobrCRLF
                DJNZ    B, OptoSt11
                RET

CmdEcho:                                ; Pro dalsi prikazy vypis prikaz na RS 
                ACALL   WriteChar       ; Zobrazi vykonavany prikaz
                MOV     A, CMDchar      ; Obnovi prikaz
RelayOn:
                CJNE    A, #'R', RelayOff
                CLR     P1.0
                RET
RelayOff:
                CJNE    A, #'r', ULN
                SETB    P1.0
                RET
ULN:
                CJNE    A, #'U', ULN_Vyp
                MOV     R0, #0
                MOV     B, #05          ; 5 bitu
ULN0:
                MOV     A, R0           ; Odrotuj R0 o 1 bit doleva
                RL      A
                MOV     R0, A
                ACALL   RxSer           ; Nacti bit do A
                CJNE    A, #'0', ULN1
                SJMP    ULN2
ULN1:
                CJNE    A, #'1', InvalInput
                INC     R0
ULN2:
                DJNZ    B, ULN0
                MOV     A, R0           ; O.K.
                SJMP    ULN_OK          ; Zapis do Stat174 a vlastni 174

ULN_Vyp:
                CJNE    A, #'V', ULN_Zap
                MOV     R0, #0FFh       ; Podle R0 se potom rozlisi Vyp/Zap
                SJMP    ULN_Com
ULN_Zap:
                CJNE    A, #'Z', Zaves
                MOV     R0, #00h
ULN_Com:                                ; Spolecny kod pro nacteni Vyp i Zap
                ACALL   RxSer           ; Nacti cislo bitu v ASCII do A
                CLR     C
                SUBB    A, #'1'         ; '1' .. '5'
                JC      InvalInput      ; C -> A < 30h -> chyba
                MOV     B, A            ; Uloz cislo bitu do B (pro cykl)
                SUBB    A, #05h         ; '5' - '1' + 1; C vynulovan
                JNC     InvalInput      ; Nebyl prenos -> A > '5'

                MOV     A, #01h
ULN_Com0:       RL      A
                DJNZ    B, ULN_Com0     ; Odrotuj bit podle B
                CJNE    R0, #0h, ULN_ComV ; 00h = Zap, 0FFh = Vyp
ULN_ComZ:
                ORL     A, Stat174      ; Zapnout = nastavit na 1
                SJMP    ULN_OK
ULN_ComV:
                CPL     A                
                ANL     A, Stat174      ; Vypnout = nastavit na 0
ULN_OK:
                MOV     Stat174, A      ; Zapis do 174
                CALL    Out174
                MOV     DPTR, #Zapsano174 ; Informace o zapsanych datech
		CALL    WrStr0
                MOV     Stat174, A
                AJMP    ZobrA

InvalInput:                             ; Chybny vstupni retezec
                MOV     DPTR, #ErrInput ; Zobrazi String "Chybne zadani"
		JMP     WrStr0

UndefCmd:                               ; Zobraz retezec pro Nedef Prikaz
                MOV     DPTR, #UndefStr
       		JMP     WrStr0          ; Zpet do MainCekej

Zaves:                                  ; Zaveseni modemu
                CJNE    A, #'$', Cas
                MOV     DPTR, #AreYouSure ; Zobraz Dotaz
                CALL	WrStr0
                ACALL   RxSer
                ANL     A, #UpCase      ; Korekce Mala -> Velka
                CJNE    A, #'A', ZavesNo
ZavesYES:                               ; Je to 'A'  
                AJMP    ZavesModem
ZavesNo:
                RET

Cas:                                    ; Neznamy prikaz nebo Konfigurace
                PUSH    Acc             ; Uloz prijaty znak
                ACALL   ZobrSpace
                ACALL   ZobrDATE
                ACALL   ZobrSpace
                ACALL   ZobrTIME
                POP     Acc
                CJNE    A, #'#', UndefCmd

Config:
                MOV     DPTR, #CfgHelp  ; Zobraz Help pro Konfiguraci
		CALL    WrStr0
                ACALL   RxSer
                MOV     B, A            ; Uloz prikaz
                ACALL   WriteChar       ; Zobraz prikaz, nici A !
                MOV     A, B            ; A znovu obnov

                CJNE    A, #'E', TestSet; Pouze 'E', ignoruj 'e'
                AJMP    EraseLog        ; Smaz Log Udalosti
TestSet:
                CJNE    A, #'S', UndefCmd; Musi byt velke 'S'
                ACALL   RxSer
                CJNE    A, #'T', TestDate ; Jeste to muze byt SET DATE
                ACALL   WriteChar       ; V A je 'T', "Echuj"
                ACALL   ZobrSpace

                ACALL   NactiBISpace    ; NactiBCD, ale Ignoruj Mezery
                JB      F0, ConfigErr
                MOV     Hour, A
                ACALL   ZobrA
                CALL    ZobrTsep

                CALL    NactiBITsep     ; NactiBCD, ale Ignoruj TIMEsep
                JB      F0, ConfigErr
                MOV     Min, A
                ACALL   ZobrA
                CALL    ZobrTsep

                CALL    NactiBITsep     ; NactiBCD, ale Ignoruj TIMEsep
                JB      F0, ConfigErr
                MOV     Sec, A
                ACALL   ZobrA

                MOV     R1, Year     ; Prestup. rok - pro zjednoduseni proc.
                SJMP    UpDateQ

ConfigErr:      SJMP    InvalInput

TestDate:
                CJNE    A, #'D', UndefCmd
                ACALL   WriteChar       ; V A je 'D', "Echuj"
                ACALL   ZobrSpace

                CALL    NactiBISpace    ; NactiBCD, ale Ignoruj Mezery
                JB      F0, ConfigErr
                MOV     Day, A
                ACALL   ZobrA
                CALL    ZobrDsep

                CALL    NactiBIDsep     ; NactiBCD, ale Ignoruj DATEsep
                JB      F0, ConfigErr
                MOV     Month, A
                ACALL   ZobrA
                CALL    ZobrDsep

                CALL    NactiBIDsep     ; NactiBCD, ale Ignoruj DATEsep
                JB      F0, ConfigErr

                MOV     R0, A           ; Nastavovany rok v BCD do R0
                ACALL   ZobrA           ; Nici A
                MOV     A, R0
                ACALL   Prestupny       ; Vypocti Posledni prestupny rok
                MOV     R1, A           ; a uloz jej do R1 (BCD)
                CALL    BCD_HEX         ; a B (HEX)
                MOV     B, A
                MOV     A, R0           ; Nastavovany rok
                CALL    BCD_HEX         ; do HEX
                CLR     C
                SUBB    A, B
                ANL     A, #03h         ; Jen posledni 2 bity
                RR      A
                RR      A               ; Rok jsou prvni dva MSB bity
                MOV     R0, A
                MOV     A, Day
                ANL     A, #3Fh
                ADD     A, R0
                MOV     Day, A

UpDateQ:
                MOV     DPTR, #AreYouSure ; Zobraz Dotaz
		CALL    WrStr0
                ACALL   RxSer
                ANL     A, #UpCase      ; Korekce Mala -> Velka
                CJNE    A, #'A', UpDateNO
UpDateYES:                              ; Je to 'A'
                CALL    UpDateRTC
                MOV     Year, R1        ; Zapis Posledni prestupny rok
                MOV     DPTR, #Updated  ; Potvrd provedeni
                CALL    WrStr0
SaveYear:                               ; Ulozi Aktualni rok do RAM v RTC
                MOV     A, Day
                ANL     A, #0C0h        ; YYTT DDDD
                RL      A
                RL      A               ; v A pocet let od prestupneho roku
                ADD     A, Year         ; Aktualni rok
                DA      A               ; Dekadicka korekce
                MOV     R2, #10h
                CALL    WriteRTC        ; Uloz rok do RAM
UpDateNO:       RET

Prestupny:                              ; Vypocte posledni prestupny rok
                CALL    BCD_HEX         ; Preved rok v A do HEX
                ANL     A, #0FCh        ; Odrizne posledni 2 bity
                JMP     HEX_BCD         ; Zpatky do BCD

RxSer:                                  ; Prijme znak z RS-232, ochrana TimeOutu
                PUSH    B
                MOV     B, #250         ; 250 * 20 ms = 5 sekund
RxSerLoop:
                MOV     A, #20
                CALL    DelayA          ; Po skonceni cekani je v A nula
                JB      RI, RxSerIn
                DJNZ    B, RxSerLoop
                POP     B
                SETB    F0              ; Nastav F0 - doslo k chybe
                DEC     SP              ; Odstan jedno volani -> z Config
                DEC     SP
                MOV     DPTR, #TimeOut  ; Zobraz Time Out
                JMP     WrStr0          ; Vrati se do Main

RxSerIn:                                ; Prijat
                CLR     RI
                MOV     A, SBUF         ; Precti ho
                POP     B
                CLR     F0              ; Bez chyby
                RET


NactiBISpace:                           ; NactiBCD, ale
                MOV     B, #' '         ; Ignoruj Mezery
                SJMP    NactiBCD
NactiBIDsep:                            ; NactiBCD, ale
                MOV     B, #DATEsep     ; Ignoruj oddelovac Datumu
                SJMP    NactiBCD
NactiBITsep:                            ; NactiBCD, ale
                MOV     B, #TIMEsep     ; Ignoruj oddelovac Casu

NactiBCD:                               ; Nacte do A 2 BCD cifry z RS
                ACALL   RxSer           ; Ignoruje znaky v B
                CJNE    A, B, NactiBC0
                SJMP    NactiBCD        ; Dokud se nebude lisit od B
                JB      F0, NactiBCe    ; Test, zda doslo k chybe
NactiBC0:
                CLR     C
                SUBB    A, #'0'         ; '0' = 30h, '9' = 39h
                JC      NactiBCe        ; C -> A < 30h -> chyba
                SWAP    A
                MOV     B, A            ; Horni octet
                SWAP    A
                SUBB    A, #0Ah         ; '9' - '0' + 1; C vynulovan
                JNC     NactiBCe        ; Nebyl prenos -> A > '9'

                ACALL   RxSer           ; Nacti druhy znak
                JB      F0, NactiBCe    ; Test, zda doslo k chybe
                CLR     C
                SUBB    A, #'0'         ; '0' = 30h, '9' = 39h
                JC      NactiBCe        ; C -> A < 30h -> chyba
                PUSH    Acc             ; Pro dalsi test
                ORL     A, B            ; Pricti horni octet, nenic C
                MOV     B, A            ; a uloz do B
                POP     Acc
                SUBB    A, #0Ah         ; '9' - '0' + 1; C vynulovan
                JNC     NactiBCe        ; Nebyl prenos -> A > '9'
                MOV     A, B
                RET                     ; F0 je shozen od RxSer
NactiBCe:
                SETB    F0              ; Doslo k chybe
                RET

; **************************************************************  Log Udalosti

InitLog:                                ; Inicializace Logu Udalosti
                MOV     LastLogHour, Hour; Do Logu se ulozi az pravidelna Konverze
                MOV     R2, #18h        ; #18h - LogAddrL
                CALL    ReadRTC         ; uloz do A
                MOV     LogAddrL, A     ; a potom do LogAddrL
                INC     R2              ; #19h - LogAddrH
                CALL    ReadRTC         ; uloz do A
                MOV     LogAddrH, A     ; a potom do LogAddrH
                RET

ZobrLog:                                ; Vypise Log Udalosti na RS
                SETB    RS_Flag
                MOV     R2, #MinLogL
                MOV     R3, #MinLogH    ; Od zacatku oblasti pro Log
ZobrLog0:
                CALL    ReadEE          ; Nejprve nacti hlavicku zaznamu
                CJNE    A, #0FFh, ZobrLog1    ; Test na prazdny zaznam v Logu
                MOV     A, R2           ; Prazdny zaznam -> Zobraz dalsi
                ADD     A, #16
                MOV     R2, A           ; Preskoc nasledujicich 16 bunek a 
                SJMP    ZobrLogDalsi    ; jdi na zpracovani dalsiho zaznamu

ZobrLog1:                               ; Zaznam neni prazdny ->
                MOV     R5, A           ; Uschovej nactenou hodnotu
                INC     R2

                MOV     A, R3           ; Zobraz Poradove cislo zaznamu
                SWAP    A
                MOV     B, A            ; Uloz pro prenos do nizsiho radu
                ANL     A, #00Fh        ; Nyni pripravena nejvyssi cislice

                MOV     DPTR, #TableHEX ; Zobrazi pouze nejvyssi cislici
                MOVC    A, @A+DPTR
                ACALL   WriteChar
 
                MOV     A, B
                ANL     A, #0F0h
                MOV     B, A            ; Pripraveno pro slouceni s nejnizsi 
                MOV     A, R2           ; Urizne spodni ctverici - vzdy nula
                SWAP    A
                ANL     A, #00Fh
                ADD     A, B            ; Nyni pripraveny i spodni 2 cislice
                ACALL   ZobrA
                ACALL   ZobrSpace

                MOV     A, R5           ; Do A nactenou hodnotu
                JZ      ZobrLogN        ; A = 0 -> Pokracovani zaznamu
                                        ; Zatim nic nedelej -pro dalsi pouziti

                CALL    ReadEE          ; Nacte Datum a Hodinu z Logu do R4,R5
                MOV     R4, A           ; DD.MM.YY:HH, v R5 jiz YYMM MMDD
                INC     R2              ; do R4 DDDH HHHH

                ACALL   ZobrLogDate     ; Zobrazi Datum a Hodinu z R4, R5

                MOV     B, #7           ; 7 okruhu primo
                MOV     DPTR, #0        ; 8. pres DPTR
ZobrLog2:
                CALL    ReadEE
                MOV     R4, A           ; Neplatne bity si orizne ZobrTMP
                ANL     A, #0C0h        ; 2 bity 8, okruhu
                RL      A
                RL      A
                ACALL   RL2_DPTR        ; Posun DPTR (8. okruh) o 2 bity
                ORL     A, DPL          ; Doplni bity do DPTR
                MOV     DPL, A

                INC     R2
                CALL    ReadEE          ; TempL
                MOV     R5, A
                INC     R2

                MOV     R0, #04         ; @R0 ukazuje na R4
                ACALL   ZobrTMP         ; Zobrazi Teplotu @R0, tj. R4 a R5
                ACALL   ZobrSpace
                DJNZ    B, ZobrLog2       

                ACALL   ZobrTDPTR       ; Zobrazi Teplotu v DPTR (8. okruh)
ZobrLogN:                               ; Dalsi zaznam s CR LF
                ACALL   ZobrCRLF
ZobrLogDalsi:                           ; Dalsi zaznam bez CR LF
                CJNE    R2, #00, ZobrLog0 ; 0 -> doslo k prenosu
                INC     R3
                CJNE    R3, #MaxLogH, ZobrLog0; Jeste neni vypsana cela pamet
                RET

ZobrLogDate:                            ; Zobrazi Datum a Hodinu z R4, R5
                MOV     A, R4
                ANL     A, #0E0h        ; Odrizne Den
                SWAP    A
                RR      A
                MOV     B, A
                MOV     A, R5           ; YYMM MMDD
                ANL     A, #03h         ; Odrizne Den
                RL      A
                RL      A
                RL      A
                ADD     A, B
                CALL    HEX_BCD
                ACALL   ZobrA           ; Zobrazi Den
                ACALL   ZobrDsep

                MOV     A, R5           ; YYMM MMDD
                ANL     A, #00111100b   ; Odrizne Mesic
                RR      A
                RR      A
                CALL    HEX_BCD
                ACALL   ZobrA           ; Zobrazi Mesic
                ACALL   ZobrDsep

                MOV     A, R5           ; YYMM MMDD
                ANL     A, #11000000b   ; Odrizne Rok
                RL      A
                RL      A
                ADD     A, Year         ; Posledni prestupny rok v BCD
                DA      A
                ACALL   ZobrA           ; Zobrazi Rok
                ACALL   ZobrTsep

                MOV     A, R4           ; DDDH HHHH
                ANL     A, #1Fh         ; Odrizne Hodinu
                CALL    HEX_BCD
                ACALL   ZobrA           ; Zobrazi Hodinu
                AJMP    ZobrSpace


ZobrEEListing:                          ; Vypise Listing EEprom na RS
                SETB    RS_Flag
                MOV     R2, #00h
                MOV     R3, #00h        ; Od uplneho zacatku
ZobrEEL0:
                MOV     A, R3           ; Nejprve Adresa
                ACALL   ZobrA 
                MOV     A, R2
                ACALL   ZobrA
                ACALL   ZobrSpace
                ACALL   ZobrSpace

                MOV     B, #16
ZobrEEL1:
                CALL    ReadEE
                ACALL   ZobrA
                ACALL   ZobrSpace
                INC     R2
                DJNZ    B, ZobrEEL1       

                ACALL   ZobrCRLF
                CJNE    R2, #00, ZobrEEL2 ; 0 -> doslo k prenosu
                INC     R3
ZobrEEL2:
                CJNE    R3, #MaxLogH, ZobrEEL0; Jeste neni vypsana cela pamet 
                RET

UpDateLog:                              ; Ulozi do Logu Aktualni teplotu
                MOV     R3, LogAddrH
                MOV     R2, LogAddrL

                MOV     A, Month        ; WWWT MMMM
                ANL     A, #1Fh 
                CALL    BCD_HEX         ; Prevede do HEX
                RL      A
                RL      A
                MOV     B, A

                MOV     A, Day          ; YYTT DDDD
                ANL     A, #11000000b   ; Odtrhne Rok
                ORL     A, B
                MOV     B, A

                MOV     A, Day          ; YYTT DDDD
                ANL     A, #3Fh         ; Odtrhne Den
                CALL    BCD_HEX         ; Prevede do HEX
                PUSH    Acc             ; Pro dalsi pouziti
                ANL     A, #18h         ; 2 nejvyssi platne bity
                RL      A
                SWAP    A               ; Odpovida 3x RR A 
                ORL     A, B                          
                CALL    WriteEE         ; YYMM MMDD
                INC     R2

                POP     Acc             ; Zkonvertovany Day
                ANL     A, #07h         ; Dolni 3 bity
                SWAP    A
                RL      A
                MOV     B, A

                MOV     A, Hour         ; xxTT HHHH
                CALL    BCD_HEX
                ORL     A, B
                CALL    WriteEE         ; DDDH HHHH
                INC     R2

                MOV     Okruh, #07      ; Teplotu 8. okruhu ulozi do DPTR
                MOV     R1, #00h        ; Zajima mne TempH = nulovy posuv
                CALL    OffsetSA        ; Do R0 ulozi adresu teploty 8. okruhu
                MOV     DPH, @R0
                INC     R0
                MOV     DPL, @R0
                ACALL   RL2_DPTR        ; Odrotuje nejvyssi 2 neplatne bity

                MOV     Okruh, #00
                MOV     B, #7           ; 7 okruhu primo
UpDateL0:
                MOV     R1, #00h        ; Zajima mne TempH = nulovy posuv
                CALL    OffsetSA        ; Do R0 ulozi adresu podle Okruhu
                MOV     A, @R0
                ANL     A, #3Fh         ; Odrizne 2 nejvyssi = neplatne bity
                MOV     @R0, A          ; Odlozi zpatky
                MOV     A, DPH
                ANL     A, #0C0h        ; 2 platne bity 8. okruhu
                ORL     A, @R0          ; Pricte TempH zpracovavaneho okruhu
                CALL    WriteEE         ; Zapise do Logu
                ACALL   RL2_DPTR        ; Pripravi nove platne bity v DPTR

                INC     R2
                INC     R0
                MOV     A, @R0
                CALL    WriteEE
                INC     R2
                INC     Okruh
                DJNZ    B, UpDateL0

                MOV     A, LogAddrL     ; Inkrementace LogAddr 
                ADD     A, #16          ; 16 Byte v jednom zaznamu
                MOV     LogAddrL, A
                MOV     A, LogAddrH
                ADDC    A, #00          ; Pricte prenos
                MOV     LogAddrH, A

                CJNE    A, #MaxLogH, ZapisLogAddr ; Test horniho konce         
                SJMP    LogOdZacatku    ; LogAddr na zacatek a Uloz do RTC

EraseLog:                               ; Vymaze Log Udalosti
                MOV     DPTR, #AreYouSure ; Zobraz Dotaz, RS_Flag nastaven pri
		ACALL   WrStr0            ; detekci prikazu
                ACALL   RxSer      ; Ochrana Timeoutem, vraci se o uroven vyse
                ANL     A, #UpCase      ; Korekce Mala -> Velka
                CJNE    A, #'A', EraseNO
EraseYES:                               ; Je to 'A'
                CALL    ClearEE         ; Smaze obsah CELE EEprom
                MOV     DPTR, #Erased   ; Potvrd provedeni
                ACALL   WrStr0

LogOdZacatku:
                MOV     LogAddrL,#MinLogL; LogAddr na zacatek a Uloz do RTC
                MOV     LogAddrH,#MinLogH
ZapisLogAddr:                           ; Zapis LogAddr do RTC
                MOV     A, LogAddrL
                MOV     R2, #18h        ; LogAddrL
                CALL    WriteRTC        ; uloz do RTC

                MOV     A, LogAddrH
                MOV     R2, #19h        ; LogAddrH
                JMP     WriteRTC        ; uloz do RTC
EraseNO:        RET


RL2_DPTR:                               ; Posune DPTR o 2 bity vlevo
                ACALL   RL_DPTR
RL_DPTR:                                ; Posune DPTR o 1 bit vlevo
                XCH     A, DPL          ; Dolni Byte, soucasne ulozi Acc
                CLR     C               ; Doplnuj nulami
                RLC     A
                XCH     A, DPL          ; vlastne XCH DPL, A - ulozi zpet
                XCH     A, DPH          ; Horni Byte
                RLC     A
                XCH     A, DPH          ; Soucasne obnovi puvodni obsah Acc
                RET

; ************************************************************  Obsluha Modemu

InitModem:                              ; Inicializace modemu
                MOV     Logged, #00h    ; Nikdo neni prilogovan
                SETB    RS_Flag
                MOV     B, #3
InitMod0:
                MOV     DPTR, #ModemAT  ; Posli AT pro synchronizaci
                ACALL   WrStr0
                CLR     A
                CALL    DelayA
                DJNZ    B, InitMod0

                MOV     DPTR, #ModemInit; Posli Inicializacni retezec
		AJMP    WrStr0

ZavesModem:
                MOV     Logged, #00h      ; Nikdo neni prilogovan
                MOV     DPTR, #ModemBreak ; Posli +++ na RS
                ACALL   WrStr0
                CLR     A                 ; Cekej
                CALL    DelayA
                CALL    DelayA
                MOV     DPTR, #ModemHang  ; Posli ATH0 na RS
		AJMP    WrStr0
       
; ****************************************************************************

ZobrDATE:                               ; Zobrazi DATUM na pozici kurzoru
                MOV     A, Day
                ANL     A, #3Fh         ; YYTT DDDD
                ACALL   ZobrA
                ACALL   ZobrDsep
                MOV     A, Month
                ANL     A, #1Fh         ; WWWT MMMM
                ACALL   ZobrA
                ACALL   ZobrDsep
                MOV     A, Day
                ANL     A, #0C0h        ; YYTT DDDD
                RL      A
                RL      A
                ADD     A, Year         ; Posledni prestupny rok
                DA      A
                ACALL   ZobrA
                RET

ZobrTIME:
                MOV     A, Hour
                ACALL   ZobrA
                ACALL   ZobrTsep
                MOV     A, Min
                ACALL   ZobrA
                ACALL   ZobrTsep
                MOV     A, Sec
                ACALL   ZobrA
                RET

ZobrCurT:                               ; Zobrazi Aktualni tepl. (dle RS_Flag)
                JB      RS_Flag, ZobrCurR
                MOV     A, #08Eh
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
                MOV     B, #6
ZobrCurL:       ACALL   ZobrSpace       ; Zobrazeni na LCD
                DJNZ    B, ZobrCurL
                SJMP    ZobrCur1
ZobrCurR:       MOV     DPTR, #CurStr   ; Zobrazeni na RS-232
                ACALL   WrStr0
                ACALL   ZobrSpace
ZobrCur1:
                MOV     R1, #00h        ; Zajima mne TempH = nulovy posuv

ZobrCmM:                                ; Zobrazi Teplotu CmM podle posuvu R1
                MOV     Okruh, #00
                MOV     B, #PocetOkruhu
ZobrCmMC:
                CALL    OffsetSA        ; Naplni R0 Adresou podle Okruhu a R1
                JB      RS_Flag, ZobrCmM1
                MOV     DPTR, #PoziceOkr; Urceni pozice pro zobrazeni teploty
                MOV     A, Okruh
                ANL     A, #07h
                MOVC    A, @A+DPTR
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
ZobrCmM1:
                ACALL   ZobrTMP         ; Zobrazi Aktualni teplotu @R0
                JNB     RS_Flag, ZobrCmM2
                ACALL   ZobrSpace       ; Jen pro RS
ZobrCmM2:
                INC     Okruh
                DJNZ    B, ZobrCmMC
                RET

ZobrMinT:                               ; Zobrazi Min teplotu (podle RS_Flag)
                JB      RS_Flag, ZobrMin0
                MOV     A, #091h
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
ZobrMin0:       MOV     DPTR, #MinStr
                ACALL   WrStr0
                JNB     RS_Flag, ZobrMin1
                ACALL   ZobrSpace       ; U RS pridej Mezeru
ZobrMin1:
                MOV     R1, #MinTempH-TempH     ; Zajima mne MinTempH
                SJMP    ZobrCmM

ZobrMaxT:                               ; Zobrazi Max teplotu (podle RS_Flag)
                JB      RS_Flag, ZobrMax0
                MOV     A, #091h
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
ZobrMax0:       MOV     DPTR, #MaxStr
                ACALL   WrStr0
                JNB     RS_Flag, ZobrMax1
                ACALL   ZobrSpace       ; U RS pridej Mezeru
ZobrMax1:
                MOV     R1, #MaxTempH-TempH     ; Zajima mne MaxTempH
                SJMP    ZobrCmM


ZobrRSDetail:                           ; Zobrazi Detailni informace na RS
                SETB    RS_Flag
                MOV     Okruh, #00
                MOV     B, #PocetOkruhu
ZobrRSDC:
                MOV     R1, #00h        ; Zajima mne TempH = nulovy posuv
                CALL    OffsetSA        ; Naplni R0 Adresou podle Okruhu
                MOV     A, Okruh
                ACALL   ZobrA
                MOV     DPTR, #CurStrD
                ACALL   WrStr0

                ACALL   ZobrTMP         ; Zobraz Aktualni teplotu @R0; R0 + 2
                MOV     DPTR, #MinStrD
                ACALL   WrStr0

                ACALL   ZobrTMP         ; Zobraz Min teplotu @R0;R0 zvysi o 2
                ACALL   ZobrSpace
                MOV     A, #'@'
                ACALL   WriteChar       ; Kdy bylo dosazeno
                ACALL   ZobrSpace
                CALL    OffsetRTC       ; Naplni R2 Adresou podle Okruhu
                ACALL   ZobrMMTime      ; Zobrazi Datum a Cas
                MOV     DPTR, #MaxStrD
                ACALL   WrStr0

                ACALL   ZobrTMP         ; Zobrazi Max teplotu @R0
                ACALL   ZobrSpace
                MOV     A, #'@'
                ACALL   WriteChar       ; Kdy bylo dosazeno
                ACALL   ZobrSpace
                CALL    OffsetRTC       ; Naplni R2 Adresou podle Okruhu
                MOV     A, R2           ; Pricet Offset Max - Min
                ADD     A, #06
                MOV     R2, A
                ACALL   ZobrMMTime      ; Zobrazi Datum a Cas

                ACALL   ZobrCRLF
                INC     Okruh
                DJNZ    B, ZobrRSDC
                RET

ZobrRSROM:                              ; Zobrazi LaseredROM na RS
                CLR     C               ; Nejprve pockej na dokonceni Konverze
                MOV     A, Counter
                SUBB    A, #10          ; 10 * 50 ms = 500 ms
                JC      ZobrRSROM       ; Byl C => A < 10 => Cekej

                SETB    RS_Flag
                MOV     Okruh, #00
                MOV     B, #PocetOkruhu
ZobrRSR0:
                MOV     A, Okruh        ; Zobraz cislo Okruhu
                ACALL   ZobrA
                MOV     A, #':'
                ACALL   WriteChar

                CALL    ReadROM         ; Nacti ROM pro dany Okruh
                ACALL   ZobrROMBuff     ; a zobraz

                MOV     NewNode, #0     ; Inicializace SearchROM
NextSearch:
                CALL    SearchROM       ; Jeden pruchod Search ROM
                ACALL   ZobrROMBuff     ; a zobraz
                MOV     A, NewNode
                JNZ     NextSearch      ; 0 v NewNode = Konec hledani

                ACALL   ZobrCRLF
                INC     Okruh
                DJNZ    B, ZobrRSR0
                RET

ZobrROMBuff:                            ; Zobrazi ROMBuff
                ACALL   ZobrSpace       ; Nici A, R0 a R1
                MOV     R0, #ROMBuff
                MOV     R1, #9
ZobrRB0:
                MOV     A, @R0
                ACALL   ZobrA
                INC     R0
                DJNZ    R1, ZobrRB0
                RET

ZobrMMTime:
                MOV     A, R2           ; Do R3 ulozi zakladni Adresu
                MOV     R3, A
                ADD     A, #04
                MOV     R2, A
                CALL    ReadRTC         ; MinDay do A
                ANL     A, #3Fh         ; YYTT DDDD
                ACALL   ZobrA
                ACALL   ZobrDsep
                INC     R2
                CALL    ReadRTC         ; MinMonth do A
                ANL     A, #1Fh         ; WWWT MMMM
                ACALL   ZobrA
                ACALL   ZobrSpace
                MOV     A, R3
                ADD     A, #03
                MOV     R2, A
                CALL    ReadRTC         ; MinHour do A
                ACALL   ZobrA
                ACALL   ZobrTsep
                DEC     R2
                CALL    ReadRTC         ; MinMin do A
                AJMP    ZobrA

ZobrCopyright:                          ; Zobraz Copyright podle RS_Flag
                JB      RS_Flag, ZobrCop0
                CALL    ClrLCD
                CALL    CursorOff
ZobrCop0:
                MOV     DPTR, #Copyright; String Copyrightu
                ACALL   WrStr0

                JB      RS_Flag, ZobrCop1
                MOV     A, #0D4h
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
                SJMP    ZobrCop2
ZobrCop1:
                ACALL   ZobrCRLF        ; Na novy radek
                MOV     DPTR,#PoslZapStr; String Posledniho Zapnuti
                ACALL   WrStr0
ZobrCop2:
                MOV     R2, #15h        ; #15h - Day
                CALL    ReadRTC         ; uloz do A
                ANL     A, #3Fh         ; YYTT DDDD
                ACALL   ZobrA
                ACALL   ZobrDsep
                INC     R2              ; #16h - Month
                CALL    ReadRTC         ; uloz do A
                ANL     A, #1Fh         ; WWWT MMMM
                ACALL   ZobrA
                ACALL   ZobrDsep
                INC     R2              ; #17h - Year - neni nutne dopocitat
                CALL    ReadRTC         ; uloz do A
                ACALL   ZobrA
                ACALL   ZobrSpace       ; Predevsim pro RS

                JB      RS_Flag, ZobrCop3
                MOV     A, #0E0h
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
ZobrCop3:
                MOV     R2, #14h        ; #14h - Hour
                CALL    ReadRTC         ; uloz do A
                ACALL   ZobrA
                ACALL   ZobrTsep
                DEC     R2              ; #13h - Min
                CALL    ReadRTC         ; uloz do A
                ACALL   ZobrA
                ACALL   ZobrTsep
                DEC     R2              ; #12h - Sec
                CALL    ReadRTC         ; uloz do A
                ACALL   ZobrA

                JB      RS_Flag, ZobrCop4
                MOV     A, #0DDh        ; Zobraz pocet zapnuti
                CALL    MoveCur         ; Vcetne shozeni RS_Flag
                SJMP    ZobrCop5
ZobrCop4:
                ACALL   ZobrCRLF        ; Na novy radek
                MOV     DPTR,#PocetZapStr; String Poctu Zapnuti
                ACALL   WrStr0
ZobrCop5:
                MOV     R2, #11h        ; Pocet zapnuti
                CALL    ReadRTC         ; uloz do A
                ACALL   ZobrA           ; v HEX

                JNB     RS_Flag, ZobrCopEnd
                ACALL   ZobrCRLF        ; Prejdi na novy radek
ZobrCopEnd:
                RET 

; ============================================================================

ZobrTMP:                                ; Zobrazi teplotu na adrese @R0
                PUSH    DPL             ; R0 zvysi o 2
                PUSH    DPH
                MOV     DPH, @R0        ; TempH
                INC     R0
                MOV     DPL, @R0        ; TempL
                INC     R0
                SJMP    ZobrTmp0

ZobrTDPTR:                              ; Zobrazi teplotu v DPTR
                PUSH    DPL
                PUSH    DPH
ZobrTmp0:
                PUSH    B
                MOV     A, DPH
                ANL     A, #3Fh         ; Orizni horni 2 neplatne bity
                MOV     DPH, A
                CJNE    A, #InvalTMP, ZobrTmp1 ; Test neplatne teploty
                MOV     DPTR, #InvalStr ; Retezec pro nedef. teplotu
                ACALL   WrStr0
                SJMP    ZobrTmpZ
ZobrTmp1:
                ACALL   DPdiv10         ; Setiny v BCD
                MOV     R7, A           ; Zbytek z Setin
                ACALL   DPdiv10         ; Desetiny v BCD
                SWAP    A
                ADD     A, R7
                MOV     R7, A           ; Zbytek z Desetin a Setin

                ACALL   DPdiv10         ; Jednotky v BCD
                MOV     R6, A           ; Zbytek z Jednotek
                ACALL   DPdiv10         ; Desitky v BCD
                SWAP    A
                ADD     A, R6
                MOV     R6, A           ; Zbytek z Desitek a Jednotek

                ACALL   DPdiv10         ; Stovky v BCD
                MOV     B, A            ; Zbytek ze Stovek

                MOV     A, R6
                SUBB    A, #50h         ; Korekce Posunute nuly
                JNC     ZobrTmp2
                DEC     B               ; Prenos -> sniz Stovky
                SUBB    A, #5Fh         ; Proved dekadickou korekci, ! C = 1
ZobrTmp2:
                MOV     R6, A           ; V R5 bud 1, 0 nebo FFh pro zaporne

                MOV     A, B
                CJNE    A, #1, ZobrTmp3 ; 1 = Teplota pres 100 stupnu
                MOV     A, #'1'
                ACALL   WriteChar       ; Zobraz 1
                SJMP    ZobrTmpD
ZobrTmp3:       JZ      ZobrTmpP        ; 0 = Kladna Teplota
                ACALL   ZobrMinus       ; Vse ostatni = Pod nulou

                CLR     C               ; Vypocet Absolutni hodnoty
                MOV     A, #99h         ; Spravne se ma odecitat od 10000, ale
                SUBB    A, R7           ; u 9999 - R6/7 + 1 nedojde k prenosu
                ADD     A, #1
                DA      A
                MOV     R7, A
                PUSH    PSW             ; Uschovej si Carry

                CLR     C               ; Vynuluj Carry
                MOV     A, #99h
                SUBB    A, R6
                POP     PSW             ; Obnov Carry
                ADDC    A, #0           ; a pricti ho
                DA      A
                MOV     R6, A
                SJMP    ZobrTmpD

ZobrTmpP:       ACALL   ZobrSpace
ZobrTmpD:                               ; Zobraz Teplotu
                MOV     A, R6
                ACALL   ZobrA           ; Zobrazi stupne
                MOV     A, #DECpoint
                ACALL   WriteChar
                MOV     A, R7
                ACALL   ZobrA           ; Zobrazi desetiny a setiny stupne
ZobrTmpZ:
;                MOV     A, #11011111b   ; Zobrazi znacku pro stupen (LCD)
                JNB     RS_Flag, ZobrTmpE
                MOV     A, #0F8h        ; Zobrazi znacku pro stupen (852)
                ACALL   WriteChar
ZobrTmpE:
                POP     B
                POP     DPH
                POP     DPL
                RET

DPdiv10:                             ; Vydeli DPTR deseti, zbytek ulozi do A
                PUSH    B
                MOV     A, DPH
                MOV     B, #10
                DIV     AB           ; A:Vysledek, B:Zbytek
                MOV     DPH, A       ; MSB vyresen

                MOV     A, DPL
                ANL     A, #0F0h     ; Nyni Zbytek + Horni octet z LSB
                ADD     A, B         ; B < 10
                SWAP    A            ; V A Zbytek + Horni octet
                MOV     B, #10
                DIV     AB        ; V Dolnim octetu A Horni octet LSB vysledku
                SWAP    A
                XCH     A, DPL       ; Mezivysledek do DPL, do A LSB

                ANL     A, #00Fh     ; Nyni Dolni octet LSB
                SWAP    A            ; Platna data B jsou take v Dolnim octetu
                ADD     A, B         ; Odpovida XCH A,B; SWAP A; ADD A,B
                SWAP    A            ; Prohod octety do spravneho poradi, tj.
                MOV     B, #10       ; Zbytek z B v Hornim + Dolni octet LSB
                DIV     AB
                ADD     A, DPL       ; Pricti mezivysledek
                MOV     DPL, A
                MOV     A, B         ; V B zbytek, presun jej do A
                POP     B
                RET

; ********************************************************  Zobrazovaci rutiny

WriteChar:                              ; Zobrazi znak v A na LCD nebo RS
                JB      RS_Flag, TxSer
                JMP     WrD_LCD
TxSer:                                  ; Odesle znak na Seriavou linku
                JNB     TI, $           ; Cekej na odvysilani predchoziho znaku
                CLR     TI
                MOV     SBUF, A
                RET

ZobrA:                                  ; Zobrazi obsah A v HEX
                PUSH    DPL
                PUSH    DPH
                MOV     DPTR, #TableHEX
                PUSH    Acc
                SWAP	A
                ANL     A, #0Fh
                MOVC    A, @A+DPTR
                ACALL   WriteChar
                POP     Acc
                ANL     A, #0Fh
                MOVC    A, @A+DPTR
                ACALL   WriteChar
                POP     DPH
                POP     DPL
                RET

ZobrSpace:                              ; Zobrazi Mezeru
                PUSH    Acc
                MOV     A, #' '
ZobrComm:
                ACALL   WriteChar
                POP     Acc
                RET

ZobrTsep:                               ; Zobrazi TIMEsep
                PUSH    Acc
                MOV     A, #TIMEsep
                SJMP    ZobrComm

ZobrDsep:                               ; Zobrazi DATEsep
                PUSH    Acc
                MOV     A, #DATEsep
                SJMP    ZobrComm

ZobrMinus:                              ; Zobrazi '-'
                PUSH    Acc
                MOV     A, #'-'
                SJMP    ZobrComm


ZobrCRLF:                               ; Zobrazi CRLF (ma smysl jen pro Ser)
                PUSH    Acc
                MOV     A, #13
                ACALL   WriteChar
                MOV     A, #10
                SJMP    ZobrComm

WrStr0:                                 ; Zobrazi String 0
                PUSH    Acc             ; String muze byt i prazdny
                SJMP    WrS1
WrS2:           ACALL   WriteChar
                INC     DPTR
WrS1:           CLR     A
                MOVC    A, @A+DPTR
                JNZ     WrS2
                POP     Acc
                RET

; ********************************************************  Obsluha Klavesnice

KBDin           EQU     0B2h            ; P3.2 = INT0

TestKBD:
                PUSH    Acc
                MOV     Kbd, #00h
                ORL     P1, #0F8h       ; 1111 1xxx
                JNB     KBDin, TestKbdE ; Chyba nebo Preruseni od RTC
                CLR     P1.3
                NOP
                JB      KBDin, TestKbd4
                SETB    P1.3
                ORL     Kbd, #01h
TestKbd4:       CLR     P1.4
                NOP
                JB      KBDin, TestKbd5
                SETB    P1.4
                ORL     Kbd, #02h
TestKbd5:       CLR     P1.5
                NOP
                JB      KBDin, TestKbd6
                SETB    P1.5
                ORL     Kbd, #04h
TestKbd6:       CLR     P1.6
                NOP
                JB      KBDin, TestKbd7
                SETB    P1.6
                ORL     Kbd, #08h
TestKbd7:       CLR     P1.7
                NOP
                JB      KBDin, TestKbdE
                SETB    P1.7
                ORL     Kbd, #10h
TestKbdE:
                POP     Acc
                RET

; ***************************************************  Obsluha 1-Wire sbernice

ReadTMP:                                ; Precte teplotu a ulozi do @R0
                PUSH    B               ; Nici mj. R2, R3, R4, R5 a R7
                ACALL   PrevedOkruh     ; Priprav Bit Portu (OkruhCnv)
                ACALL   Read_SPad       ; Spolecne pro DS1820 i DS1822
                JB      F0, ReadTMPe    ; Nastaven F0 = doslo k chybe (CRC)
                CALL    Prev1820        ; Preved teplotu pro DS1820
                SJMP    ReadTMPc
ReadTMPe:
                MOV     @R0, #InvalTMP  ; Ulozi misto chybne hodnoty InvalTMP
                INC     R0
                MOV     @R0, #InvalTMP
ReadTMPc:
                POP     B
                RET

PrevedOkruh:                        ; Prevede Okruh na OkruhCnv pomoci Conv125
                PUSH    DPH
                PUSH    DPL
                MOV     DPTR, #Conv125
                PUSH    Acc
                MOV     A, Okruh
                ANL     A, #07h
                MOVC    A, @A+DPTR
                MOV     OkruhCnv, A
                POP     Acc
                POP     DPL
                POP     DPH
                RET

Read_SPad:                              ; Nacte Scratch Pad do R2-R5,CRC do R7
                PUSH    B               ; pocitadlo neuspesnych cteni
                CLR     EA              ; Zakaz preruseni (casovani 1-Wire)
                MOV     R7, #00         ; Vynuluj pocatecni hodnotu pro CRC
                MOV     B, #3           ; Max. 3 pokusy
ReadSPad:
                ACALL   OneW_Reset
                MOV     A, #0CCh        ; Skip ROM
                ACALL   OneW_Tx
                MOV     A, #0BEh        ; Read Scratch Pad
                ACALL   OneW_Tx
                ACALL   OneW_Rx         ; R2 - LSB
                MOV     R2, A
                ACALL   CalcCRC
                ACALL   OneW_Rx         ; R3 - MSB - Znamenko
                MOV     R3, A
                ACALL   CalcCRC
                ACALL   OneW_Rx         ; TH
                ACALL   CalcCRC
                ACALL   OneW_Rx         ; TL
                ACALL   CalcCRC
                ACALL   OneW_Rx         ; Reserved / Configuration
                ACALL   CalcCRC
                ACALL   OneW_Rx         ; Reserved / Reserved
                ACALL   CalcCRC
                ACALL   OneW_Rx         ; R4 - Count Remain / Reserved
                MOV     R4, A
                ACALL   CalcCRC
                CALL    OneW_Rx         ; R5 - Count Per C / Reserved
                MOV     R5, A
                ACALL   CalcCRC
                ACALL   OneW_Rx         ; CRC
                ACALL   CalcCRC         ; CRC tam zapocti taky -> 0 = O.K.
                MOV     A, R7
                JZ      ReadSPOK        ; Je-li vse v poradku, je tam nula
                DJNZ    B, ReadSPad
                SETB    F0              ; Opakovana chyba CRC
                SJMP    ReadSPEnd
ReadSPOK:       CLR     F0              ; O.K. -> F0 = 0
ReadSPEnd:      SETB    EA              ; Povoli preruseni (casovani 1-Wire)
                POP     B
                RET

ReadROM:                                ; Nacte LaseredROM do ROMBuff
                ACALL   PrevedOkruh     ; Priprav Bit Portu (OkruhCnv)
                MOV     R7, #00         ; Vynuluj pocatecni hodnotu pro CRC
                MOV     R0, #ROMBuff    ; Adresa Bufferu
                MOV     R1, #8          ; Nacti celkem 8 Byte
                CLR     EA              ; Zakaz preruseni (casovani 1-Wire)
    
                ACALL   OneW_Reset
                MOV     A, #033h        ; Read ROM
                ACALL   OneW_Tx
ReadROM0:
                ACALL   OneW_Rx         ; R2 - LSB
                MOV     @R0, A
                ACALL   CalcCRC
                INC     R0
                DJNZ    R1, ReadROM0

                SETB    EA              ; Povoli preruseni (casovani 1-Wire)
                MOV     A, R7           ; Jeste uloz CRC - ma to byt 0
                MOV     @R0, A     
                RET

SearchROM:                              ; Nacte ROM do ROMBuff pomoci Search
                ACALL   PrevedOkruh     ; Priprav Bit Portu (OkruhCnv)
                ACALL   Pripni151       ; Pripne prislusny vstup 151 na P3.5
                CLR     A
                MOV     Node, A         ; Node := 0, Promenna pro cykl
                XCH     A, NewNode      ; NewNode := 0 => Koncova podminka
                MOV     OldNode, A      ; OldNode := NewNode
                CLR     EA              ; Zakaz preruseni (casovani 1-Wire)
    
                ACALL   OneW_Reset
                MOV     A, #0F0h        ; Search ROM
                ACALL   OneW_Tx
NextNode:                               ; Urci adresu zpracovavaneho bitu
                                        ; Nutne pro Minuly beh a Ulozeni bitu
                                        ; Vypocitava se z Node - 1, tj. 0..63
                MOV     A, Node         ; bity 3.-5. urcuji Byte
                RL      A               ; xxBB Bxxx -> xBBB xxxx
                SWAP    A               ; xxxx xBBB
                ANL     A, #07h         ; 0000 0BBB
                ADD     A, #ROMBuff     ; K offsetu pricti zaklad
                MOV     R0, A           ; Adresu Byte uloz do R0

                MOV     A, Node         ; bity 0.-2. urcuji bit
                ANL     A, #07h         ; 0000 0bbb
                INC     A               ; Korekce pro DJNZ
                MOV     R1, A           ; do R1 pocet rotaci pro ziskani bitu
                MOV     R7, A           ; do R7 pro DJNZ v PodleMinuleho

                INC     Node            ; Zacatek Algoritmu => Zvys cislo Node
                CLR     A
                ACALL   OneW_Rx1        ; Nacti 1. bit do C
                RLC     A               ; a uloz do A
                ACALL   OneW_Rx1        ; Nacti 2. (negovany) bit do C
                RLC     A               ; a taky uloz do A
                JNZ     UlozBit         ; 00=Kolize, 01=0, 10=1, 11=No Device

                MOV     A, Node         ; Doslo ke Kolizi
                CJNE    A, OldNode, SearchR0; Node = OldNode => bod Rozhodnuti
                MOV     A, #02h         ; V predchozim behu jsi sel po Vetvi 0
                SJMP    UlozBit         ; Nyni tedy jdi po Vetvi 1

SearchR0:       JNC     Vetev0          ; Nebyl Prenos => Node > OldNode
                                        ; => Jeste jsi zde nebyl => Vetev 0

PodleMinuleho:                          ; Vetev urci podle minuleho behu
                MOV     A, @R0          ; Hledany Byte do A
PodleMi0:       RRC     A
                DJNZ    R7, PodleMi0    ; Najdi bit a uloz jej do C

                MOV     A, #02h         ; Bude-li C => Vetev 1,skoc na ulozeni
                JC      UlozBit         ; Skoc, je-li Vetev 1,tj. 0 jiz hotova
Vetev0:
                MOV     A, Node         ; Vetev 0 => jeste se sem musis vratit
                MOV     NewNode, A      ; -> aktualizuj NewNode
                MOV     A, #01h         ; Jdi po Vetvi 0
UlozBit:
                RR      A               ; Negovany bit mne nezajima
                RRC     A               ; Do C Ukladany bit
                PUSH    PSW
                ACALL   OneW_Tx1        ; Zapis C (= platnou vetev) na 1-Wire

;                CLR     A               ; Rychlejsi, ale o 2 Byte delsi kod
;                SETB    C
;UlozBit0:       RLC     A
;                DJNZ    R1, UlozBit0    ; Odrotuj bit na spravne misto
;                MOV     R1, A           ; Uloz pro pozdejsi OR
;                CPL     A
;                ANL     A, @R0          ; Vyrizni prislusny bit
;                POP     PSW             ; Obnov PSW = Aktualni bit do C
;                JNC     UlozBit1        ; C = 0 => Hotovo
;                ORL     A, R1           ; C = 1 => ORuj
;UlozBit1:       

                MOV     A, R1
                MOV     R7, A
                MOV     A, @R0
UlozBit0:       RRC     A
                DJNZ    R1, UlozBit0    ; Odrotuj bit na spravne misto
                POP     PSW             ; Obnov PSW = Aktualni bit do C
UlozBit1:       RLC     A
                DJNZ    R7, UlozBit1    ; a Odrotuj bit zpatky

                MOV     @R0, A          ; Nakonec uloz Aktualizovany Byte

                MOV     A, Node
                CJNE    A, #64, NextNode; 64 = Jiz jsi prosel vse

                SETB    EA              ; Povoli preruseni (casovani 1-Wire)

                MOV     R7, #00         ; Vynuluj pocatecni hodnotu pro CRC
                MOV     R0, #ROMBuff    ; Pocatecni adresa Bufferu
                MOV     R1, #08         ; Celkem pro 8 Byte
SearchCRC:
                MOV     A, @R0          ; Jeste vypocti CRC z nactene ROM
                ACALL   CalcCRC
                INC     R0
                DJNZ    R1, SearchCRC

                MOV     A, R7           ; CRC uloz - ma to byt 0
                MOV     @R0, A
                RET

Start_Conv:                             ; Zada prikaz pro start Konverze
                CLR     EA              ; Zakaz preruseni (casovani 1-Wire)
                MOV     OkruhCnv, #0FFh ; Vsechny okruhy
                ACALL   OneW_Reset
                MOV     A, #0CCh        ; Skip ROM
                ACALL   OneW_Tx
                MOV     A, #044h        ; Convert
                ACALL   OneW_Tx
DQ_PullUp:                              ; Vsechny okruhy najednou
                MOV     P0, #0FFh       ; "1"
                MOV     P2, #00h        ; Enable 125
                ACALL   SetTimer0       ; Nastavi Timer0 a povoli Preruseni
                RET

CalcCRC:                                ; Vypocte CRC z obsahu A
                PUSH    Acc
                PUSH    B
                PUSH    Acc             ; Ulozeni Vypocitavaneho Byte
                MOV     B, #8           ; 8 bitu = 8 cyklu
CalcCRC1:
                XRL     A, R7           ; v R7 predchozi mezivysledek CRC
                RRC     A               ; XOR vstupniho bitu do C
                MOV     A, R7           ; predchozi vysledek CRC do A
                JNC     CalcCRC2        ; A xor 0 = A, nemeni se -> preskoc
                XRL     A, #18h         ; 18 vyplyva z Polynomu
CalcCRC2:
                RRC     A               ; Odrotuj, novy vstupni bit v C
                MOV     R7, A           ; Uloz novou hodnotu mezivysledku CRC
                POP     Acc             ; Obnova Vypocitavaneho Byte
                RR      A               ; Posun bity ve Vypocitavanem Byte
                PUSH    Acc             ; a uloz zpet do zasobniku
                DJNZ    B, CalcCRC1     ; cele opakuj pro vsech 8 bitu
                POP     Acc
                POP     B
                POP     Acc
                RET

OneW_Tx:                                ; Zapise Byte na 1-Wire sbernici
                PUSH    B
                MOV     B, #8           ; 8 bitu
OW_TxLoop:
                RRC     A
                ACALL   OneW_Tx1        ; Vysle bit v C na 1-Wire sbernici
                DJNZ    B, OW_TxLoop

                POP     B
                RET

OneW_Tx1:                               ; Zapise C (1 Bit) na 1-Wire sbernici
                PUSH    Acc
                JNC     OW_Tx0          ; Skoc podle bitu C
                ACALL   DQ_01           ; Zapis 0 a ihned 1
                SJMP    OW_TxWait
OW_Tx0:
                ACALL   DQ_0            ; Zapis 0 a potom jako data opet 0
OW_TxWait:      MOV     A, #5           ; Vysilaci cyklus je 60 uS
                ACALL   A10Wait
                ACALL   DQ_1            ; Min. 1 uS v 1
                POP     Acc
                RET

OneW_Rx:                                ; Precte Byte z 1-Wire sbernice
                ACALL   Pripni151       ; Pripne spravny vstup 151 na P3.5
                PUSH    B
                MOV     B, #8           ; 8 bitu
OW_RxLoop:
                ACALL   OneW_Rx1        ; Do C Bit z 1-Wire sbernice
                RRC     A               ; A neni nutne predtim mazat
                DJNZ    B, OW_RxLoop
                POP     B
                RET

OneW_Rx1:                               ; Precte 1 Bit z 1-Wire sbernice
                PUSH    Acc

                ACALL   DQ_01           ; Nastavi DQ na 0 a ihned na 1
                MOV     A, #5           ; Nyni cekej na platny Bit
                DJNZ    Acc, $          ; Cekej celkem 13 Cyklu, vcetne RET
                MOV     C, P3.5         ; Precti Bit

                MOV     A, #5           ; Neni nutne uchovavat C, protoze 
                ACALL   A10Wait         ; A10Wait nenici PSW
                POP     Acc
                RET

OneW_Reset:                             ; Provede RESET 1-Wire sbernice
                PUSH    Acc
                ACALL   DQ_0
                MOV     A, #48+1        ; tRSTL - min. 480 nS v "0"
                ACALL   A10Wait
                ACALL   DQ_1
                MOV     A, #7           ; Min. tPDHIGH + Min. tPDLOW
                ACALL   A10Wait
                                        ; Zde by bylo Cteni Presence Pulse
                MOV     A, #48-7+1      ; tRSTH - min. 480 nS v "1"
                ACALL   A10Wait
                POP     Acc
                RET

DQ_0:                                   ; Shodi DQ na 0 
                MOV     A, OkruhCnv     ; Nesmi nicit C, ale A nicit muze
                CPL     A
                ANL     P0, A           ; "0"
                ANL     P2, A           ; Enable 125 = CLR bit
                RET

DQ_1:                                   ; Nastavi DQ na 1
                MOV     A, OkruhCnv     ; Nesmi nicit C, ale A nicit muze
                ORL     P2, A           ; Disable 125 = SETB bit
                RET

DQ_01:                                  ; Nastavi DQ na 0 a ihned na 1
                MOV     A, OkruhCnv     ; Nejprve posli na 1-Wire 0
                ORL     P2, A           ; Disable 125 = "1" - pro jistotu
                CPL     A
                ANL     P0, A           ; "0"
                CPL     A               ; Zase puvodni hodnotu z OkruhCnv

                XRL     P2, A           ; Enable 125 = "0"
                                        ; xor : 0 = zustava, 1 = negace
                ORL     P2, A           ; Disable 125 = "1"
                RET

Pripni151:                              ; Pripni spravny vstup 151
                ANL     P1, #03         ; Nici A
                MOV     A, Okruh
                ANL     A, #07          ; Max. 8 okruhu
                SWAP    A
                ORL     P1, A
                RET

; ----------------------------------------------------------------------------

Prev1820:                        ; Prevede teplotu s max. presnosti pro DS1820
;
;                          PerC - Remain  
; TMP = TEMP_READ - 0.25 + -------------
;                              PerC
;
                MOV     A, R3           ; MSB
                RRC     A               ; v C je znamenko
                MOV     A, R2           ; LSB
                RRC     A               ; Odstran Pulstupne
                ADD     A, #50          ; Posunuta nula - 50 stupnu

                MOV     B, #10          ; Preved na setiny stupnu Celsia
                MUL     AB              ; Vynasob 100x a uloz do DP
                MOV     DPH, B          ; A:LSB, B:MSB
                MOV     B, #10          ; v A je LSB
                MUL     AB
                MOV     DPL, A          ; Vysledne LSB
                PUSH    B               ; Uloz prenos do vyssiho radu
                MOV     A, DPH
                MOV     B, #10
                MUL     AB
                POP     B               ; MSB zahod,do B prenos z nizsiho radu
                ADD     A, B            ; Pricti prenos z nizsiho radu
                MOV     DPH, A          ; TEMP_READ v DP

                CLR     C               ; Odecti 0.25 stupne
                MOV     A, DPL
                SUBB    A, #25
                MOV     DPL, A
                JNC     PrevedT0
                DEC     DPH             ; Byl prenos, odecti 1 z vyssiho radu

PrevedT0:
                MOV     A, R4           ; Remain; Je-li 0 => Podil = 1 => 0,99
                JZ      Preved255
                MOV     A, R5           ; PerC
                MOV     B, A            ; B := PerC
                CLR     C
                SUBB    A, R4           ; Remain; A := PerC - Remain
                ACALL   A_Div_B         ; A := (A / B)
                SJMP    PrevedSetiny
Preved255:
                MOV     A, #255
PrevedSetiny:                           ; v A nyni stupne v 1/256
                MOV     B, #100       ; preved je do 1/100 vynasobenim 100/256
                MUL     AB
                MOV     A, B            ; Vezmu-li MSB -> vydelil jsem 256
                ADD     A, DPL          ; Pricti k DP
                MOV     DPL, A
                JNC     BezPrenosu
                INC     DPH             ; Byl prenos -> Zvys stupne
BezPrenosu:                             ; Uloz DP do @R0 
                MOV     @R0, DPH        ; (TempH)
                INC     R0
                MOV     @R0, DPL        ; (TempL)
                DEC     R0
                RET


A_Div_B:                                ; A deleno B v deset. HEX, A < B
                                        ; Nici R4, R5 a R7           
                MOV     R4, #00         ; Vysledek
                MOV     R5, B           ; Delitel - Pomocne ulozeni
                MOV     R7, #08         ; Citac - 8 bitu
A_DivLoop:      RL      A               ; Vynasob 2
                MOV     B, R5
                DIV     AB
                JZ      A_Div1          ; Je to delitelne ?
                SETB    C
A_Div1:         MOV     A, R4
                RLC     A               ; Pripoj k vysledku
                MOV     R4, A
                MOV     A, B            ; Zbytek se pokus dal delit
                DJNZ    R7, A_DivLoop   ; Opakuj 8x

                MOV     A, R4
                RET

CompTMP:                                ; Porovna TempH,L s Max a Min
                CJNE    @R0, #InvalTMP, CompTmp1
                RET
CompTmp1:
                PUSH    Acc
                MOV     A, R0
                ADD     A, #04          ; Offset MaxTempH
                MOV     R1, A

                CJNE    @R1, #InvalTMP, CompTmp2
                ACALL   UpDateMax       ; Test na nedef. teplotu Max, Min
                DEC     R1
                DEC     R1
                CJNE    @R1, #InvalTMP, PorovMin
                ACALL   UpDateMin
                SJMP    KonecPorov
CompTmp2:
                MOV     A, @R0          ; (TempH)
                CLR     C
                SUBB    A, @R1          ; (MaxTempH)
                JC      PorovMin        ; Je mensi -> skoc
                JNZ     MaxVyssi        ; <> 0 -> TempH > MaxTempH
                INC     R0
                MOV     A, @R0          ; (TempL) TempH = MaxTempH
                DEC     R0
                INC     R1
                SUBB    A, @R1          ; (MaxTempL)
                JC      PorovMin        ; Je mensi -> skoc
                JZ      PorovMin        ; Je stejny -> skoc
MaxVyssi:
                ACALL   UpDateMax       ; Je vyssi -> UpDate

PorovMin:
                MOV     A, R0
                ADD     A, #MinTempH-TempH
                MOV     R1, A

                MOV     A, @R0          ; (TempH)
                CLR     C
                SUBB    A, @R1          ; (MinTempH)
                JC      MinMensi        ; Je mensi -> skoc
                JNZ     KonecPorov      ; <> 0 -> TempH > MinTempH
                INC     R0
                MOV     A, @R0          ; (TempL) TempH = MinTempH
                DEC     R0
                INC     R1
                SUBB    A, @R1          ; (MinTempL)
                JNC     KonecPorov      ; Neni mensi -> skoc
MinMensi:
                ACALL   UpDateMin
KonecPorov:     POP     Acc
                RET

UpDateMax:                              ; Zkopiruje aktualni teplotu a cas
                PUSH    Acc
                MOV     A, R0
                ADD     A, #MaxTempH-TempH
                MOV     R1, A           ; Do R1 adresa MaxTempH v RAM
                ACALL   OffsetRTC       ; Do R2 adresa MinTempH v RTC
                MOV     A, R2
                ADD     A, #06          ; Potrebuji MaxTempH
                MOV     R2, A
                SJMP    UpDateMM        ; Spolecna cast UpDate Min a Max

UpDateMin:                              ; Zkopiruje aktualni teplotu a cas
                PUSH    Acc
                MOV     A, R0
                ADD     A, #MinTempH-TempH
                MOV     R1, A           ; Do R1 adresa MinTempH v RAM
                ACALL   OffsetRTC       ; Do R2 adresa MinTempH v RTC
UpDateMM:
                MOV     A, @R0          ; (TempH)
                MOV     @R1, A          ; (MinTempH)
                ACALL   WriteRTC        ; Zapis MinTempH
                INC     R0
                INC     R1
                INC     R2
                MOV     A, @R0          ; (TempL)
                DEC     R0
                MOV     @R1, A          ; (MinTempL)
                ACALL   WriteRTC        ; Zapis MinTempL
                INC     R2
                MOV     A, Min
                ACALL   WriteRTC        ; Zapis Minuty
                INC     R2
                MOV     A, Hour
                ACALL   WriteRTC        ; Zapis Minuty
                INC     R2
                MOV     A, Day
                ACALL   WriteRTC        ; Zapis Minuty
                INC     R2
                MOV     A, Month
                ACALL   WriteRTC        ; Zapis Minuty
                POP     Acc
                RET

OffsetRTC:                              ; Vypocte Offset v RAM pameti v RTC
                PUSH    Acc             ; bunky MinTempH a ulozi jej do R2
                PUSH    B
                MOV     A, Okruh
                MOV     B, #12
                MUL     AB              ; Rel. adresa bunky MinTempH senzoru
                ADD     A, #20h         ; Absolutni adresa
                MOV     R2, A           ; do R2
                POP     B
                POP     Acc
                RET

OffsetSA:                               ; Vypocte Offset v poli SenzorArray
                PUSH    Acc             ; bunky TempH+R1 a ulozi jej do R0
                PUSH    B
                MOV     A, Okruh
                MOV     B, #6
                MUL     AB              ; Relativni adresa bunky TempH senzoru
                ADD     A, #SenArray    ; Absolutni adresa
                ADD     A, R1
                MOV     R0, A           ; do R0
                POP     B
                POP     Acc
                RET

; ***************************************************************  Obsluha I2C

WRcmdEE         EQU     10101110b       ; Prikaz pro zapis do EEprom
RDcmdEE         EQU     10101111b       ; 24C02, 24C32 a dalsi
WRcmdRTC        EQU     10100000b       ; Prikaz pro zapis do RTC
RDcmdRTC        EQU     10100001b

SCL             EQU     0B7h            ; P3.7 - Clock
SDA             EQU     0B6h            ; P3.6 - Data

RTC:                                    ; Precte cas z RTC
                MOV     R2, #02         ; Sekundy
                ACALL   ReadRTC
                MOV     Sec, A
                INC     R2
                ACALL   ReadRTC
                MOV     Min, A
                INC     R2
                ACALL   ReadRTC
                MOV     Hour, A
                INC     R2

                ACALL   ReadRTC
                MOV     Day, A
                INC     R2
                ACALL   ReadRTC
                MOV     Month, A
                RET

UpDateRTC:                              ; Zapise aktualni cas do RTC
                MOV     R2, #02         ; Sekundy
                MOV     A, Sec
                ACALL   WriteRTC
                INC     R2
                MOV     A, Min
                ACALL   WriteRTC
                INC     R2
                MOV     A, Hour
                ACALL   WriteRTC
                INC     R2

                MOV     A, Day
                ACALL   WriteRTC
                INC     R2
                MOV     A, Month
                AJMP    WriteRTC

ReadEE:                                 ; Precte Byte z EEprom, Adr. v R2 a R3
                MOV     A, #WRcmdEE     ; Nyni je horni cast adresy pripravena
                ACALL   I2C_TxS         ; Posli na I2C se Start Cond.
                MOV     A, R3           ;; Zapis horni byte Adresy
                ACALL   I2C_Tx          ;;
                MOV     A, R2           ; Zapis spodni byte Adresy
                ACALL   I2C_Tx

                MOV     A, #RDcmdEE     ; Prikaz pro cteni dat
                ACALL   I2C_TxS
                ACALL   I2C_Rx
                AJMP    I2C_Stop        ; STOP condition

WriteEE:                                ; Zapise Byte do EEprom, Adr. v R2 a R3
                PUSH    Acc
                MOV     A, #WRcmdEE     ; Nyni je horni cast adresy pripravena
                ACALL   I2C_TxS
                MOV     A, R3           ;; Zapis horni byte Adresy
                ACALL   I2C_Tx          ;;
                MOV     A, R2           ; Zapis spodni byte Adresy
                ACALL   I2C_Tx

                POP     Acc             ; Data
                ACALL   I2C_Tx          ; Posli je
                ACALL   I2C_Stop        ; STOP condition
                MOV     A, #10
                AJMP    DelayA          ; 10 ms na zapsani

ClearEE:                                ; Smaze EEprom, nici A, B, R2 a R3
                MOV     R3, #MaxLogH    ; Pocet stranek pameti
ClearEE0:
                MOV     R2, #256/PageSize ; Pocet zapisovych stranek v stran-
                                          ; kach pameti
ClearEE1:                               ; Pamet se maze od konce po zapisovych
                MOV     A, #WRcmdEE     ; strankach
                ACALL   I2C_TxS
                MOV     A, R3           ;; Zapis horni byte Adresy
                DEC     A               ;; Pro cyklus je R3 o jednicku vetsi
                ACALL   I2C_Tx          ;;
                MOV     A, R2           ; R2-1 je vlastne por. cislo zapisove stranky
                DEC     A
                MOV     B, #PageSize    ; Velikost zapisove stranky
                MUL     AB              ; V A nyni spodni byte adresy
                ACALL   I2C_Tx          ; Zapis adresu zacatku stranky
                MOV     B, #PageSize    ; Velikost zapisove stranky               
ClearEE2:
                MOV     A, #0FFh        ; Zapisovana hodnota
                ACALL   I2C_Tx
                DJNZ    B, ClearEE2

                ACALL   I2C_Stop        ; STOP condition
                MOV     A, #10
                ACALL   DelayA          ; 10 ms na zapsani
                DJNZ    R2, ClearEE1

                DJNZ    R3, ClearEE0
                RET

ReadRTC:                                ; Precte Byte z RTC
                MOV     A, #WRcmdRTC    ; Nejprve zapis Adresu
                ACALL   I2C_TxS
                MOV     A, R2           ; Adresa
                ACALL   I2C_Tx

                MOV     A, #RDcmdRTC    ; Prikaz pro cteni dat
                ACALL   I2C_TxS
                ACALL   I2C_Rx
                AJMP    I2C_Stop        ; STOP condition

WriteRTC:                               ; Zapise Byte do RTC
                PUSH    Acc
                MOV     A, #WRcmdRTC    ; Nejprve zapis Adresu
                ACALL   I2C_TxS
                MOV     A, R2           ; Adresa
                ACALL   I2C_Tx
                POP     Acc             ; Data
                ACALL   I2C_Tx          ; Posli je
                AJMP    I2C_Stop        ; STOP condition

I2C_TxS:                                ; Posle data do I2C se START cond.
                SETB    SDA             ; Data  = 1
                SETB    SCL             ; Clock = 1
                NOP
                NOP
                NOP
                NOP
                CLR     SDA             ; START - Data do 0
                NOP
                NOP
                NOP
                NOP
                CLR     SCL             ; Clock do 0

I2C_Tx:                                 ; Posle data do I2C bez START cond.
                PUSH    B
                MOV     B, #8           ; 8 bitu
I2C_TxLoop:     RLC     A
                JNC     I2C_Tx0
                SETB    SDA
                SJMP    I2C_Tx2
I2C_Tx0:        CLR     SDA
I2C_Tx2:        SETB    SCL
                NOP
                NOP
                NOP
                NOP
                CLR     SCL
                DJNZ    B, I2C_TxLoop
                SETB    SDA
                NOP
                NOP
                SETB    SCL
                NOP
                NOP
                NOP
                NOP
                CLR     SCL
                POP     B
                RET

I2C_Rx:                                 ; Precte Byte z I2C do A
                PUSH    B
                MOV     B, #8           ; 8 bitu
                SETB    SDA
I2C_RxLoop:     CLR     SCL
                NOP
                NOP
                NOP
                NOP
                NOP
                SETB    SCL
                CLR     C
                JNB     SDA , I2C_Rx2
                CPL     C
I2C_Rx2:        RLC     A
                DJNZ    B, I2C_RxLoop
                CLR     SCL
                POP     B
                RET

I2C_Stop:                               ; Vysle STOP cond. na I2C
                CLR     SDA
                NOP
                NOP
                NOP
                NOP
                SETB    SCL
                NOP
                NOP
                NOP
                NOP
                SETB    SDA
                RET

; ***************************************************************  Obsluha LCD

InitLCD:                                ; Inicializace LCD Displaye
                MOV     A, #20          ; Min. 15 ms po RESETu
                ACALL   DelayA
                MOV     P1, #33h
                ACALL   Pulse
                MOV     A, #5
                ACALL   DelayA
                ACALL   Pulse
                ACALL   Delay
                ACALL   Pulse

                MOV     P1, #23h
                ACALL   Pulse
                ACALL   Delay
                MOV     A, #28h
                ACALL   WrI_LCD
                ACALL   Delay

                MOV     A, #08h         ; Vypni Display
                ACALL   WrI_LCD
ClrLCD:                                 ; Provede Reset = Vymazani LCD
                MOV     A, #01h         ; Reset Displaye
                ACALL   WrI_LCD
                MOV     A, #02
                ACALL   DelayA          ; Reset trva 1,64 ms
                MOV     A, #06h
                ACALL   WrI_LCD

                MOV     A, #0Eh
                SJMP    WrI_LCD         ; Vcetne RET

MoveCur:                                ; Presune kurzor na pozici v A
                CLR     RS_Flag         ; Prepne vystup na LCD
MoveCur2:
                ACALL   WrI_LCD
                AJMP    Delay           ; Cekaci rutina

CursorOff:                              ; Cursor Off
                MOV     A, #0Ch
                SJMP    MoveCur2

CursorOn:                               ; Cursor On
                MOV     A, #0Fh
                SJMP    MoveCur2

WrI_LCD:                                ; Zapise Instrukci v A do LCD
                PUSH    Acc
                ANL     A, #0F0h
                ANL     P1, #03h
                ORL     P1, A
                ACALL   Pulse
                POP     Acc
                SWAP	A
                ANL     A, #0F0h
                ANL     P1, #03h
                ORL     P1, A
                ACALL   Pulse
                JMP     Wait40

WrD_LCD:                                ; Zapise Data v A do LCD
                PUSH    Acc
                ANL     A, #0F0h
                ORL     A, #08h
                ANL     P1, #03h
                ORL     P1, A
                ACALL   Pulse
                POP     Acc
                SWAP	A
                ANL     A, #0F0h
                ORL     A, #08h
                ANL     P1, #03h
                ORL     P1, A
                ACALL   Pulse
                JMP     Wait40

Pulse:                                  ; Zapisovaci Impuls
                SETB    P1.2
                NOP                     ; Delka Impulsu min. 450 ns
                ACALL   Wait40
                CLR     P1.2
                RET

; ************************************************************  Pomocne rutiny

Out174:                                 ; Zapise obsah Stat174 do 174 = ULN
                PUSH    Acc
                MOV     A, Stat174
                ANL     A, #1Fh         ; jen 5 bitu
                RL      A
                RL      A
                RL      A               ; D7 - D3
                ANL     P1, #01h        ; E (LCD) = 0, En (174) = 0
                ORL     P1, A           ; Priprav data na piny D7 - D3
                NOP
                SETB    P1.1            ; Zapise do 174
                NOP
                CLR     P1.1
                POP     Acc
                RET

SetTimer0:                              ; Povoli Casovac 0 a preruseni
                MOV     TH0, #TH0_Val   ; Naplneni Casovace 0
                MOV     TL0, #TL0_Val
                MOV     Counter, #00    ; Naplni promennou
                SETB    EA              ; Povoli vsechna preruseni
                RET

A10Wait:                                ; Ceka A * 10 uS
                NOP
                NOP
                NOP
                NOP
                NOP
                NOP
                NOP
                DJNZ    Acc, A10Wait
                RET

Wait40:                                 ; Ceka cca 40 mikrosekund
                PUSH    Acc
                MOV     A, #14
                DJNZ    Acc, $
                POP     Acc
                RET

DelayA:                                 ; Ceka A * Delay = A milisekund
                ACALL   Delay
                DJNZ    Acc, DelayA
                RET

Delay:				        ; Ceka asi 1 ms / 11.0592 MHz
		PUSH	PSW             ; Celkem 921 vcetne CALL a RET
                PUSH    Acc             ;   7 + 227 * 4 + 6
		MOV	A, #227
DelayLoop:	NOP
		NOP
		DJNZ	Acc, DelayLoop
		POP	Acc
		POP	PSW
		RET

BCD_HEX:                                ; Prevede BCD cislo do HEX
                PUSH    B
                PUSH    Acc
                ANL     A, #0Fh         ; jednotky do R0
                MOV     B, A
                POP     Acc
                ANL     A, #0F0h
                SWAP    A               ; desitky
                PUSH    B
                MOV     B, #10
                MUL     AB              ; Nizsi v A
                POP     B
                ADD     A, B            ; Pricti jednotky
                POP     B
                RET

HEX_BCD:                                ; Prevede HEX cislo do BCD
                PUSH    B
                MOV     B, #10
                DIV     AB              ; A = A div 10, B = A mod 10
                SWAP    A               ; A desitky
                ADD     A, B            ; pricti jednotky B
                POP     B
                RET


; *****************************************************************  Konstanty

Copyright:				; Retezec s Copyrightem
		DB	'TMS  ver. '
                DB      MainVer
		DB      '.'
                DW      SubVer
                DB      '      (c), Pavel Kaspar'
		DB	00h

Help:
                DB      13, 10
;                DB      CMDprefix, '? - Help', 13, 10
                DB      CMDprefix, 'c - Aktualni teploty', 13, 10
                DB      CMDprefix, 'M - Max teploty', 13, 10
                DB      CMDprefix, 'm - Min teploty', 13, 10
                DB      CMDprefix, 'd - Detailni vypis teplot', 13, 10
                DB      CMDprefix, 'n - Vypis s/n senzoru', 13, 10
                DB      CMDprefix, 'l - Vypis Log udalosti', 13, 10
                DB      CMDprefix, 'L - Vypis EEPROM', 13, 10
                DB      CMDprefix, 's - Zobraz stav vstupu', 13,10
;                DB      CMDprefix, 'S - Zobraz 10x stav vstupu', 13,10
;                DB      CMDprefix, 'R - Zapni rele', 13, 10
;                DB      CMDprefix, 'r - Vypni rele', 13, 10
                DB      CMDprefix, 'U - Zapis 5 bitu na vystup', 13, 10
                DB      CMDprefix, 'Z - Zapni n-ty vystup', 13, 10
                DB      CMDprefix, 'V - Vypni n-ty vystup', 13, 10
                DB      CMDprefix, '$ - Zaves modem', 13, 10
                DB      CMDprefix, '# - Konfigurace', 13, 10, 0

CfgHelp:        DB      13, 10, '** Konfigurace **', 13, 10
                DB      'E - Vymaz Log udalosti', 13, 10
                DB      'ST HH', TIMEsep, 'MM', TIMEsep, 'SS'
                DB      ' - Nastav Cas', 13, 10
                DB      'SD DD', DATEsep, 'MM', DATEsep, 'YY'
                DB      ' - Nastav Datum', 13, 10, 0

PoslZapStr:     DB      'Posledni zapnuti : ', 0
PocetZapStr:    DB      'Pocet zapnuti    : ', 0

AreYouSure:     DB      13, 10, 'Jste si jist ? (A/N)', 13, 10, 0
Updated:        DB      'Zmeneno ...', 13, 10, 0
Erased:         DB      'Vymazano ...', 13, 10, 0
Zapsano174:     DB      ' Na vystup ULN bylo zapsano ', 0

UndefStr:       DB      '  Neznamy prikaz, zadej ', CMDprefix, '?'
                DB      ' pro napovedu', 0

TimeOut:        DB      13, 10, 'Timeout ...', 13, 10, 0

ErrInput:       DB      'Chybne zadani', 0

Opto1Str:       DB      'OPTO1=',0
Opto2Str:       DB      '  OPTO2=',0

CurStr:         DB      'Cur', 0
MinStr:         DB      'Min', 0
MaxStr:	        DB      'Max', 0

CurStrD:        DB      ':  Cur:', 0
MinStrD:        DB      '  Min:', 0
MaxStrD:        DB      '  Max:', 0        ; Pro zobrazeni Detailnich teplot

InvalStr:       DB      ' --', DECpoint, '--', 0

Conv125:        DB      80h, 20h, 08h, 02h, 01h, 04h, 10h, 40h
PoziceOkr:      DB      80h, 87h, 0C0h, 0C7h, 0CEh, 94h, 9Bh, 0A2h
TableHEX:	DB	'0123456789ABCDEF'
;DayOfWeek:      DB      'PoUtStCtPaSoNe'

ModemAT:        DB      'AT', 13, 0                       ; AT sekvence
ModemHang:      DB      'ATH0', 13, 0                     ; Zaves modem
ModemBreak:     DB      '+++', 0                          ; Break prikaz
ModemInit:      DB      'AT&FE0Q0V0X0H0S0=5S15=2M1L2'     ; Obecne
                DB      'N1&D0&K0', 13, 0                 ; pro DeskPorte

                END
